/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP.
 * GASP Demonstrator: a gasp demonstration game.
 *
 * Copyright (C) 2004 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Franck Caron, Romain Pellerin
 */
/*
 * Created on 2 december 2005
 *
 * @author CARON Franck - CNAM CEDRIC
 * @author PELLERIN Romain - CNAM CEDRIC
 * @author DELPIANO Fabien - Filao
 * 
 * 
 * Research project: CNAM-CEDRIC - INT - Filao
 */
//#include parameters.h
////#define DEMO
////#define VERSION_SOLO
////#define FRANCK
////#define LASTPING
////#define RESYNC

//#endinclude
package app2.client.game;
import javax.microedition.lcdui.Graphics;
import app2.client.NetworkCom;


public class MultiplayerActor extends Actor {

	public int multiplayerID;
	
	private final int MAX_UPDATES_DELTATIME = 3000;
	private int updatesDeltaTime = MAX_UPDATES_DELTATIME;
	private final int X_DIMENSION = 0;
	private final int Y_DIMENSION = 1;
	private int[] wantedPosition = new int[2];
	private int[] wantedPositionSpeeds = new int[2];

	public int updatesReceived = 0;
	private long drStartTime;
	private int speedAbility = 1500;
	protected static final boolean DEBUG_POS = false;
	public static final int NB_DIMENSIONS = 2;
	
	public void setX(int x1) {
		x = x1;
	}
	public void setY(int y1) {
		y = y1;
	}

	/**
	 * @param game
	 */
	public MultiplayerActor(GameCanvas game, int id) {
		super(game);
		multiplayerID = id;
	}
	
	public void initialize() {
		this.x = GameCanvas.xMaxD/2; //1000000;
		this.y = GameCanvas.yMaxD/2; //1000000;
	}

	protected int speedOfMoving = 1000;

    public boolean isPositionReached = true;
	
	public synchronized int getWantedX(){
	    return wantedPosition[X_DIMENSION];
	}
	
	public synchronized int getWantedY(){
	    return wantedPosition[Y_DIMENSION];
	}
	
	/**
	 * Sets the horizontal and vertical speeds wanted
	 * @param speedx Horizontal speed wanted
	 * @param speedy Vertical speed wanted
	 * @since 1.0
	 */
	 public synchronized void setSpeedWanted(int speedx, int speedy){
	    speedWanted[X_DIMENSION] = speedx;
	    speedWanted[Y_DIMENSION] = speedy;
	}
	
	/**
	 * Returns the horizontal speed wanted
	 * @return Horizontal speed wanted
	 * @since 1.0
	 */
	 public synchronized int getSpeedWantedX(){
	    return speedWanted[X_DIMENSION];
	}
	
	/**
	 * Returns the vertical speed wanted
	 * @return vertical speed wanted
	 * @since 1.0
	 */
	 public synchronized int getSpeedWantedY(){
	    return speedWanted[Y_DIMENSION];
	}
	
	/**
	 * Sets the horizontal speed wanted
	 * @param speedx Horizontal speed wanted
	 * @since 1.0
	 */
	public synchronized void setSpeedWantedX(int speedx){
	    speedWanted[X_DIMENSION] = speedx;
	}
	
	/**
	 * Sets the vertical speed wanted
	 * @param speedy Vertical speed wanted
	 * @since 1.0
	 */
	public synchronized void setSpeedWantedY(int speedy){
	    speedWanted[Y_DIMENSION] = speedy;
	}
	
	 /**
	 * Sets the horizontal and vertical speeds wanted
	 * @param speedx Horizontal speed wanted
	 * @param speedy Vertical speed wanted
	 * @since 1.0
	 */
	
	public synchronized void setWantedPosition(int x, int y){
	    wantedPosition[X_DIMENSION] = x;
	    wantedPosition[Y_DIMENSION] = y;
	}
	
	public synchronized int getWantedPositionSpeedX(){
	    return wantedPositionSpeeds[X_DIMENSION];
	}
	
	public synchronized int getWantedPositionSpeedY(){
	    return wantedPositionSpeeds[Y_DIMENSION];
	}
	
	public synchronized void setWantedPositionSpeeds(int speedX, int speedY){
	    wantedPositionSpeeds[X_DIMENSION] = speedX;
	    wantedPositionSpeeds[Y_DIMENSION] = speedY;
	}
	
	/*********************/
	/****** Actions ******/
	/*********************/
	
	// outdistance compared to the position of destination from which  
	// the actor is readjusted on its destination
	private static final int recalArea4DR = 2000;
	/**
	 * Move the multiplayer actor towards the position specified 
	 * in the last update of position received, use of the dead reckoning(DR)
	 * "smooth back"
	 */
	public synchronized boolean moveTowardsWithDR(long deltaTime){
    
	    if (isPositionReached){
	    	if (System.currentTimeMillis() < (drStartTime + (updatesDeltaTime/2))) 
	                return true;
	        
	        setSpeedWanted(getWantedPositionSpeedX(),getWantedPositionSpeedY());
	    }
	    else{
	        // apply the same moving algorithm for multiplayer
	        // using dead reckoning
			int diff_x = getWantedX() - getX();
			int diff_y =  getWantedY() - getY();
			int speedx=0, speedy=0;
			
			// speedx 
			if (Math.abs(diff_x) <= recalArea4DR) 
				setX(getWantedX());	
			else{
				if (diff_x < 0){
					speedx = -speedAbility;
				}  
				else if (diff_x > 0){
					speedx = speedAbility;
				}
			}
			
			// speedy 
			if (Math.abs(diff_y) <= recalArea4DR)
				setY(getWantedY());
			else{ 
			    if (diff_y < 0){
					speedy = -speedAbility;
				}
				else if (diff_y > 0) {
					speedy = speedAbility;
				}
			}
			
			// position wanted reached
			if (speedx==0 && speedy==0){
				isPositionReached = true;
				drStartTime = System.currentTimeMillis();
			}
						
			// set speeds
			setSpeedWanted(speedx,speedy);
	    }

	    wantToHaveSpecificSpeed = true;
	    return true;
	}

	public int speedWanted[] = new int[NB_DIMENSIONS];
	public boolean wantToHaveSpecificSpeed = false;
	protected void applyDecisionsWithoutAnyConstraints() {
		if (wantToHaveSpecificSpeed)
			setSpeed(speedWanted[0], speedWanted[1]);
	}
	
	
	public void executeActions(long time, long delta_time) {
		moveTowardsWithDR(delta_time);
		applyDecisionsWithoutAnyConstraints();
		super.executeActions(time, delta_time);
	}
	
	/*********************/
	/****** Graphic ******/
	/*********************/

	public void paint(Graphics g, long gameTime, long deltaTime) {
		super.paint(g, gameTime, deltaTime);
		g.setColor(0x3333FF);
		g.drawString(username, game.convertToScreenX(getX()), game.convertToScreenY(getY())+5, Graphics.TOP | Graphics.HCENTER);
	}

	/*****************************/
	/*** Collisions management ***/
	/*****************************/

	/**
	 * This part is to manage collision between actor and/or other objects. It's not include in this demostrator.
	 */

}
