/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * MooDS: a communication protocol used in GASP.
 * GASP Demonstrator: a gasp demonstration game.
 *
 * Copyright (C) 2004 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Franck Caron, Romain Pellerin
 */
/*
 * Created on 2 december 2005
 *
 * @author CARON Franck - CNAM CEDRIC
 * @author PELLERIN Romain - CNAM CEDRIC
 * 
 * 
 * Research project: CNAM-CEDRIC - INT - Filao
 */
package app2.server;

//import java.util.Enumeration;

import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.log4j.Category;
import org.mega.gasp.event.DataEvent;
import org.mega.gasp.event.EndEvent;
import org.mega.gasp.event.JoinEvent;
import org.mega.gasp.event.QuitEvent;
import org.mega.gasp.event.StartEvent;
import org.mega.gasp.event.impl.DataEventImpl;
import org.mega.gasp.server.GASPServer;

import app2.NewPlayer;


/**
 * @author PELLERIN Romain (CNAM Paris, romain.pellerin@laposte.net)
 * @author DELPIANO Fabien (Pastagames, fabien@pastaga.net)
 * @author KOCIK Fabrice - (Pastagames, fabrice@pastis.org)
 * @author CARON Franck - (CNAM Paris)
 * --
 * JIMM Project 2005
 * (Bouygues Telecom, CNAM Paris, Pastagames)
 */
public class Server extends GASPServer {

	public static Category cat;

	private int maxActor = 0;

	private boolean gameIsStarted = false;
	
	
	/**
	 * This table allow us to know if a player is in game or not.
	 * players[aSID]=1 if the player is connected, else -1.
	 */

	private Hashtable playersTable = new Hashtable();
	
	
	public Server() {

		super();
		init();

		cat = Category.getInstance("GASPLogging");
		cat.debug("Log4J Server Category instanciated");

	}

	/**
	 * Initialization of the fields of the server.
	 */
	private void init() {
	}

	public void start() {
		cat.debug("Server START running!");
	}

	public void end() {
		cat.debug("Server STOP!");
	}

	/*********************************************************************************/
	/** Listenners : onStartEvent, onEndEvent, onJoinEvent, onQuitEvent, onDataEvent**/
	/*********************************************************************************/

	public void onStartEvent(StartEvent se) {
		cat.debug("STARTEVENT received by server: aSID>"
				+ se.getActorSessionID());
		start();
		gameIsStarted = true;
	}

	public void onEndEvent(EndEvent ee) {
		cat.debug("ENDEVENT received by server: aSID>"
				+ ee.getActorSessionID());
		end();
	}

	public void onJoinEvent(JoinEvent je) {
		cat.debug("JOINEVENT received by server: aSID>"
				+ je.getActorSessionID() + " username>" + je.getUsername());

		maxActor++;
	}
	
	public void onQuitEvent(QuitEvent qe) {
		cat.debug("QUITEVENT received by server: aSID>"
				+ qe.getActorSessionID());

		// for waiting room only
		if (!gameIsStarted)
			maxActor--;
		
		int aSID = qe.getActorSessionID();
		if (gameIsStarted) {
			playersTable.remove(""+aSID);
		}

	}

	public void onDataEvent(DataEvent de) {
		cat.debug("DATAEVENT received by server: aSID>"
				+ de.getActorSessionID() + " Hashtable size:"
				+ de.getData().size());

		Hashtable h1 = de.getData();

		int nbObj = h1.size();

		for (int i = 0; i < nbObj; i++) {
			 Object o = (Object) h1.get(i + "");
			 treatMessage(o, de.getActorSessionID());
		 }
	}
	
	/**
	 * Treatment of Messages object received.
	 * 
	 */
	private synchronized void treatMessage(Object o, int senderASID) {
		 Player p = null;
		 if (o instanceof NewPlayer) {
			 NewPlayer np = (NewPlayer) o; 
			 cat.info("*****************************************");
			 cat.info("BEGIN of the NEW PLAYER TREATMENT");
			 cat.info("Object New Player received:");
			 cat.info("ID: " + np.getId());
			 cat.info("X: " + np.getX());
			 cat.info("Y: " + np.getY());
			 cat.info("Username :"+ np.getUsername());
			 
			 p = new Player(np.getId());
			 playersTable.put(""+np.getId(),p);
			 p.setX(np.getX());
			 p.setY(np.getY());
			 p.setUsername(np.getUsername());
			 
			 cat.info("Player informations received: " + p.getId());
			 sendInGamePlayersInfos(np.getId());
			 notifyToAllOtherPlayers(np, np.getId());
			 
			 cat.info("END of the NEW PLAYER TREATMENT");
			 cat.info("*****************************************");
		 }
		 
	 }


	/**
	 * Sending to the player (aSID) the object ErrorMessage with the type type
	 * 
	 * @param type 
	 * @param aSID
	 */
	
	private void sendErrorMessage(byte type, int aSID) {

	 cat.debug("Error message sended, type: " + type);
	 }
	
	
	/**
	 * Send to the player (idJoiner) a newPlayer message for each player in game,
	 * whose all informations were recovered.
	 * Only automatic mode
	 * 
	 * @param idJoiner
	 */
	
	private void sendInGamePlayersInfos(int idJoiner) {
		cat.debug("sendInGamePlayersInfos called");
		Hashtable h = new Hashtable();
		int nbObj = 0;
		Player p = null;
		showPlayers();
		
		// Build a newPlayer message to all players already in game and add in the hashtable.
		Enumeration enume = playersTable.keys();
		while (enume.hasMoreElements()){
			short pid = Short.parseShort((String) enume.nextElement());
			if (idJoiner != pid){
				p = (Player) playersTable.get(""+pid);
				cat.debug("playerAimed(" + idJoiner + ") playerInfos(" + pid
						+ ")");
				
				// new player message to send to idJoiner
				NewPlayer np = new NewPlayer();
				np.setId((short) p.getId());
				np.setX(p.getX());
				np.setY(p.getY());
				np.setUsername(p.getUsername());
				h.put(""+nbObj,np);
				
				nbObj++;
				cat.debug("NEW PLAYER message added to"+ idJoiner
						+ ": player n"+ p.getId());
			}
		}
	
		
		// Build a dataEvent if some newPlayer messages have to be sent.
		if (!h.isEmpty()) {
			DataEvent de = new DataEventImpl(0, h);
			cat.debug("SendDataTo player>" + idJoiner + " de>" + de);
			this.sendDataTo(idJoiner, de);
		}
	}

	/**
	 * Sending a Message Object to all players
	 * @param o
	 *  
	 */
	private void notifyToAllPlayers(Object o) {
		cat.debug("notifyAllPlayers called");
		Hashtable h = new Hashtable();
		h.put("0", o);

		DataEvent de = new DataEventImpl(0, h);
		showPlayers();

		Enumeration enume= playersTable.keys();
		while (enume.hasMoreElements()){
			short pid = Short.parseShort((String) enume.nextElement());
			this.sendDataTo(pid,de);
			if (o instanceof NewPlayer)
				cat.debug("Message NewPlayer sent to aSID");
		}
	}

	/**
	 * Sending a Message object to each player but not to the sender.
	 * 
	 * @param o, idSender
	 */
	private void notifyToAllOtherPlayers(Object o, int idSender) {
		cat.debug("notifyAllOtherPlayers called");
		Hashtable h = new Hashtable();
		h.put("0", o);

		DataEvent de = new DataEventImpl(0, h);
		showPlayers();
		
		Enumeration enume= playersTable.keys();
		while (enume.hasMoreElements()){
			short pid = Short.parseShort((String) enume.nextElement());
			if ((idSender != pid)){
				this.sendDataTo(pid,de);
				if (o instanceof NewPlayer)
					cat.debug(" Message NewPlayer sent to aSID");
			}
		}
}

	/**
	 * Sending a Message object to a specific player.
	 * 
	 * @param o, playerASID
	 */
	private void sendMessageToPlayer(Object o, int playerASID) {
		cat.debug("sendMessageToPlayer called");
		Hashtable h = new Hashtable();
		h.put("0", o);

		DataEvent de = new DataEventImpl(0, h);
		this.sendDataTo(playerASID, de);
	}
	
	
	/**
	 * Sending a Message to a specific player.
	 * 
	 * @param v, playerASID
	 */
	private void sendMessagesToPlayer(Hashtable ht /*Vector v*/, int playerASID) {
		cat.debug("sendMessageToPlayer called");
		Hashtable h = new Hashtable();
		
		int count = 0;
		Enumeration enume = ht.keys();
		while (enume.hasMoreElements()){
			short pid = Short.parseShort((String) enume.nextElement());
			h.put(""+count,ht.get(""+pid));
			count++;
		}
		DataEvent de = new DataEventImpl(0, h);
		this.sendDataTo(playerASID, de);
	}

	/**
	 * Show players state
	 *
	 */
	private void showPlayers() {
		Enumeration enume= playersTable.keys();
		while (enume.hasMoreElements()){
			short pid = Short.parseShort((String)enume.nextElement());
			cat.debug("playersTable key{"+ pid+"} = "+ playersTable.get(""+pid));
		}
	}
}
